(* :Title: Inverse Plot *)

(* :Context: RMCPackages`InversePlot` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
	This package allows the user to plot a graph of the
	inverse function to a given function over a given
	interval. 
*)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 3.0 *)

(* :History: V1.0 by Bruce F. Torrence, October 2001.*)

(* :Keywords: inverse function, inverse, function *)


BeginPackage["RMCPackages`InversePlot`"];

 
InversePlot::usage="InversePlot[f, {x, xmin, xmax}] 
takes the expression f of x and plots it in red for the 
x values between xmin and xmax, and then reflects the graph across 
the line y=x.  If this 'potential inverse' (shown in blue) is 
not the graph of a function, then f does not have a unique 
inverse on the interval from xmin to xmax.";
 
Begin["`Private`"];
       
Attributes[InversePlot]={HoldAll};

InversePlot[f_, {x_, a_, b_}, opts___Rule]:=
  Module[{fplot, pointlist, xmin, xmax,
          inverseplot, xyline},
    fplot=Plot[f, {x, a, b},
                    opts,
                    PlotStyle->Hue[1],
                    DisplayFunction->Identity];
    pointlist=Last[Graphics[fplot][[1]]][[1,1]];
    xmin=Min[Flatten[pointlist]];
    xmax=Max[Flatten[pointlist]];
    inverseplot=Graphics[{RGBColor[0, 0, 1],
                         Line[Reverse/@pointlist]}];
    xyline=Graphics[{Line[{{xmin,xmin},{xmax,xmax}}]}];
    Show[fplot,inverseplot,xyline,
            opts,
            Axes->True,
            AspectRatio->Automatic, (*1.15*)
            DisplayFunction->$DisplayFunction,
            PlotLabel->
              FontForm[
                    SequenceForm["f[x] = ",f, " shown in red."],
                    {"Courier", 12}]
           ]
    ]
            
End[];
EndPackage[];