(* by Bruce F. Torrence, Last Modified 11/24/2000 *)

BeginPackage["RMCPackages`PolarRectangles`",
	"RMCPackages`CircularPlot3D`","Utilities`FilterOptions`"]

ArcSegments::usage = "ArcSegments is an option for PolarRectangularSolid and
PolarRiemannSum. It may be set to any natural number greater than 1. It
controls the number of line segments used in representing the curved
portions of polar 'rectangular' solids. The default setting is Automatic,
which results in 10 segments for every 90 degrees of arc."

PolarRectangle::usage = "PolarRectangle[{rmin,rmax},{\[Theta]min,\[Theta]max}]
will produce a Graphics object consisting of the polar 'rectangle' with
rmin < r < rmax and \[Theta]min < \[Theta] < \[Theta]max. Use the Show command
to display the graphic. The option RectangleStyle may be set to any
Graphics directive (such as Hue[1] or Thickness[.01]), or to any list of 
such directives. PolarRectangle will also accept any Graphics
options." ;

PolarRectangle3D::usage = "PolarRectangle3D[{rmin,rmax},{\[Theta]min,\[Theta]max},h]
will produce a Graphics3D object consisting of the polar 'rectangle' in the
plane z=h with rmin < r < rmax and \[Theta]min < \[Theta] < \[Theta]max. Use 
the Show command to display the graphic. The option RectangleStyle may be set to any
Graphics3D directive (such as Hue[1] or Thickness[.01]), or to
any list of such directives. PolarRectangle3D will also accept any
Graphics3D options." ;

PolarGrid::usage = "PolarGrid[{rmin,rmax},{\[Theta]min,\[Theta]max}]
will produce a Graphics object consisting of the 10 by 10 polar grid 
with rmin < r < rmax and \[Theta]min < \[Theta] < \[Theta]max. Use 
the Show command to display the graphic. 
PolarGrid[{rmin,rmax,dr},{\[Theta]min,\[Theta]max,d\[Theta]}]
will produce the grid with radial increments of dr and angular 
increments of d\[Theta]. The option GridStyle may be set to any
Graphics directive (such as Hue[1] or Thickness[.01]), or to
any list of such directives. PolarGrid will also accept any Graphics
options." ;

PolarGrid3D::usage = "PolarGrid3D[{rmin,rmax},{\[Theta]min,\[Theta]max}]
will produce a Graphics3D object consisting of the 10 by 10 polar grid 
in the plane z=0 with rmin < r < rmax and \[Theta]min < \[Theta] < \[Theta]max. 
Use the Show command to display the graphic. 
PolarGrid3D[{rmin,rmax,dr},{\[Theta]min,\[Theta]max,d\[Theta]}]
will produce the grid with radial increments of dr and angular 
incements of d\[Theta]. The option GridStyle may be set to any
Graphics3D directive (such as Hue[1] or Thickness[.01]), or to
any list of such directives. PolarGrid3D will also accept any
Graphics3D options." ;

VerticalPolygon::usage = "VerticalPolygon[{p1,p2},{bot,top}] creates 
a Polygon (a Graphics3D object), which is the vertical rectangle 
whose bottom is the line segment joining the points p1 and p2 in the 
plane z=bot, and whose top lies directly above in the plane z=top.
The points p1 and p2 may be specified as either 2-tuples or 3-tuples. 
To view it, use the command 
Show[Graphics3D[VerticalPolygon[...]]]. Additional arguments may be
used to specify Graphics3D directives (such as EdgeForm[]).";

VerticalWall::usage = "VerticalWall[{p1,p2,...,pn},{bot,top}] creates a
list of Polyogns (Graphics3D objects), which are the vertical
rectangles whose bottom is the polygonal path joining the points 
p1, p2, ... , pn in the plane z=bot, and whose top lies directly 
above in the plane z=top. The points p1, p2, ... may be specified as 
either 2-tuples or 3-tuples. To view it, use the command 
Show[Graphics3D[VerticalWall[...]]].";

PolarRectangularSolid::usage = "PolarRectangularSolid[{rmin,rmax},
{\[Theta]min,\[Theta]max},{h1,h2}] will produce a list of Graphics3D 
polygons that make up the 'rectangular' solid from height z=h1 to
height z=h2 over the polar 'rectangle' with rmin < r < rmax and
\[Theta]min < \[Theta] < \[Theta]max. To view it, use the command
Show[Graphics3D[PolarRectangularSolid[...]]]. The option ArcSegments may
be set to any positive integer value greater than 1; it controls 
the number of line segments used in representing the curved portions
of the top and bottom polar rectangles.";

PolarSolid::usage = "PolarSolid[{rmin,rmax},
{\[Theta]min,\[Theta]max},{h1,h2}] will produce a Graphics3D object 
comprised of the polygons that make up the 'rectangular' solid from height z=h1 to
height z=h2 over the polar 'rectangle' with rmin < r < rmax and
\[Theta]min < \[Theta] < \[Theta]max. To view it, use the command
Show[PolarSolid[...]]. The option ArcSegments may
be set to any positive integer value greater than 1; it controls 
the number of line segments used in representing the curved portions
of the top and bottom polar rectangles. PolarSolid will also accept
any Graphics3D options.";

PolarRiemannSum::usage = "PolarRiemannSum[f,{r,rmin,rmax},{\[Theta],
\[Theta]min,\[Theta]max}] produces a graphic showing side by side
the polar rectangular solids in a Riemann sum, and a graph of the function
f[r,\[Theta]] over the polar grid in the x y plane. The Riemann sum is computed evaluating
the function at the center of each polar rectangle in the grid.  
PolarRiemannSum[f,{r,rmin,rmax,dr},{\[Theta],\[Theta]min,\[Theta]max,d\[Theta]}]
will produce the Riemann sum with radial increments of dr and angular 
incements of d\[Theta]. The option ArcSegments controls 
the number of line segments used in representing the curved portions
of each polar rectangular solid. It is set to 2 by default. It should be set
higher if very few rectangles are used in the grid. Any ParametricPlot3D
option may also be used; it will be passed to the graphic on the right."; 

Options[PolarRectangle]={RectangleStyle->{}};

Options[PolarRectangle3D]={RectangleStyle->{}};

Options[PolarGrid]={GridStyle->{}};

Options[PolarGrid3D]={GridStyle->{}};

Options[ArcPtList3D]={ArcSegments->Automatic};

Options[PolarRectangularSolid] = {ArcSegments->Automatic};

Options[PolarRiemannSum]={ArcSegments->2};

Begin["`Private`"]

 
PolarRectangle[{a_,b_},{tmin_,tmax_},opts___Rule]:=Module[
		{cmin=Cos[tmin],cmax=Cos[tmax],
		 smin=Sin[tmin],smax=Sin[tmax],rs
		},
		
	rs=RectangleStyle/.{opts}/.Options[PolarRectangle];	
		
	Graphics[{rs,
		Line[{{b*cmin,b*smin},{a*cmin,a*smin}}],
        Line[{{b*cmax,b*smax},{a*cmax,a*smax}}],
        Circle[{0,0},b,{tmin,tmax}],
		Circle[{0,0},a,{tmin,tmax}]
		}, FilterOptions[Graphics,opts]//Evaluate,
		   AspectRatio->Automatic]
	]
	
PolarGrid[{rmin_,rmax_,dr_:Automatic},{tmin_,tmax_,
      dt_:Automatic},opts___Rule]:=
      Module[
		{ndr=dr,ndt=dt,del=10,rvals,tvals,gs},
		
		If[dr===Automatic,ndr=(rmax-rmin)/del];
		If[dt===Automatic,ndt=(tmax-tmin)/del];
		rvals=Table[r,{r,rmin,rmax,ndr}];
		tvals=Table[t,{t,tmin,tmax,ndt}];
		gs=GridStyle/.{opts}/.Options[PolarGrid];
		
		Graphics[{gs,
		  Map[Line[{{rmin*Cos[#],rmin*Sin[#]},{rmax*Cos[#],rmax*Sin[#]}}]&,
            tvals],
          Map[Circle[{0,0},#,{tmin,tmax}]&, rvals]
          },FilterOptions[Graphics,opts]//Evaluate,
            AspectRatio->Automatic
        ]
	]  	   
	


(* no longer used...
Arc3D[r_,tmin_,tmax_,h_:0,opts___Rule]:=
  Module[{t,pplot},
    pplot=
      ParametricPlot3D[{r*Cos[t],r*Sin[t],h},{t,tmin,tmax},
        DisplayFunction->Identity,opts];
    Nest[First,pplot,1]
  ]	
 *)
   
   
ArcPtList3D[r_, tmin_, tmax_, h_:0, opts___Rule] := 
  Module[{as, dt, t}, 
    as = ArcSegments /. {opts} /. Options[ArcPtList3D]; 
    If[as === Automatic, as = 20.*(tmax - tmin)/Pi]; 
    dt = N[(tmax - tmin)/as]; 
    Table[{r*Cos[t], r*Sin[t], h}, {t, tmin, tmax, dt}]
  ] 
    

PolarRectangle3D[{a_,b_},{tmin_,tmax_},h_:0,opts___Rule]:=
	Module[
			{cmin=Cos[tmin],cmax=Cos[tmax],
			smin=Sin[tmin],smax=Sin[tmax],gs},
			rs=RectangleStyle/.{opts}/.Options[PolarRectangle3D];
    	Graphics3D[{rs,
			 Line[{{b*cmin,b*smin,h},{a*cmin,a*smin,h}}],
          	 Line[{{b*cmax,b*smax,h},{a*cmax,a*smax,h}}],
        	 Line[ArcPtList3D[a,tmin,tmax,h]],
        	 Line[ArcPtList3D[b,tmin,tmax,h]]
    	},FilterOptions[Graphics3D,opts]//Evaluate]
  	] 
  	

PolarGrid3D[{rmin_,rmax_,dr_:Automatic},{tmin_,tmax_,
      dt_:Automatic},opts___Rule]:=
      Module[
		{ndr=dr,ndt=dt,del=10,rvals,tvals,gs},
		
		If[dr===Automatic,ndr=(rmax-rmin)/del];
		If[dt===Automatic,ndt=(tmax-tmin)/del];
		rvals=Table[r,{r,rmin,rmax,ndr}];
		tvals=Table[t,{t,tmin,tmax,ndt}];
		gs=GridStyle/.{opts}/.Options[PolarGrid3D];
    
    	Graphics3D[{gs,
		  Map[Line[{{rmin*Cos[#],rmin*Sin[#],0},{rmax*Cos[#],rmax*Sin[#],0}}]&,
          			tvals],
          Map[Line[ArcPtList3D[#,tmin,tmax]]&,rvals]
          },FilterOptions[Graphics3D,opts]//Evaluate]
      ]
      

VerticalPolygon[{p1_,p2_},{bot_,top_},dir___]:=Module[{
		bottom={p1,p2}/.{x_?NumericQ,y_,z___}->{x,y,bot},
		lid={p2,p1}/.{x_?NumericQ,y_,z___}->{x,y,top}
		},
		{dir,Polygon[Join[bottom,lid]]}
	]
	
	
(* not used in this package, but good to have worked out,
a different solution is used below in PolarRectangularSolid: *)
VerticalWall[ptlist_,{bot_,top_}]:=
		Map[VerticalPolygon[#,{bot,top}]&,Partition[ptlist,2,1]]
		


PolarRectangularSolid[{a_,b_},{tmin_,tmax_},{bot_,top_},opts___]:=Module[
		{cmin=Cos[tmin],cmax=Cos[tmax],
		 smin=Sin[tmin],smax=Sin[tmax],
		 arcah,arcbh,arca0,arcb0,as},
		
		as=ArcSegments /. {opts} /. Options[PolarRectangularSolid];
				
		arca0=ArcPtList3D[a,tmin,tmax,bot,ArcSegments->as];
		arcb0=ArcPtList3D[b,tmin,tmax,bot,ArcSegments->as];
		arcah=ArcPtList3D[a,tmin,tmax,top,ArcSegments->as];
		arcbh=ArcPtList3D[b,tmin,tmax,top,ArcSegments->as];
		
   {
			
	  Map[VerticalPolygon[#,{bot,top},EdgeForm[]]&,
          MapThread[List,{Drop[arca0,-1],Drop[arca0,1]}]],
				
      Map[VerticalPolygon[#,{bot,top},EdgeForm[]]&,
          MapThread[List,{Drop[arcb0,-1],Drop[arcb0,1]}]],

      Prepend[
	    Map[Polygon,
		 Thread[
		  List[Drop[arcah,-1],Drop[arcbh,-1],Drop[arcbh,1],Drop[arcah,1]]]
	    ],EdgeForm[]
      ],

      Prepend[
	    Map[Polygon,
		 Thread[
		  List[Drop[arca0,-1],Drop[arcb0,-1],Drop[arcb0,1],Drop[arca0,1]]]
	    ],EdgeForm[]
      ],

	  VerticalPolygon[{{a*cmin,a*smin},{b*cmin,b*smin}},{bot,top}],
				
	  VerticalPolygon[{{a*cmax,a*smax},{b*cmax,b*smax}},{bot,top}],
	  
	  Line[arcah], Line[arcbh], Line[arca0], Line[arcb0]		
				
	}
		
]
			

(* No longer used...
semi-old-PolarRectangularSolid[h_,{a_,b_},{tmin_,tmax_},opts___]:=Module[
		{cmin=Cos[tmin],cmax=Cos[tmax],
		 smin=Sin[tmin],smax=Sin[tmax],
		 arcah,arcbh,arca0,arcb0,pp},
		
		ap=ArchPoints /. {opts} /. Options[PolarRectangularSolid];
		
		arcah=Arc3D[a,tmin,tmax,h,PlotPoints->ap]/.Line[{q_,w_}]->w;
		arcbh=Arc3D[b,tmin,tmax,h,PlotPoints->ap]/.Line[{q_,w_}]->w;
		arca0=Arc3D[a,tmin,tmax,0,PlotPoints->ap]/.Line[{q_,w_}]->w;
		arcb0=Arc3D[b,tmin,tmax,0,PlotPoints->ap]/.Line[{q_,w_}]->w;
		
   {
			
	  Map[VerticalPolygon[#,h,EdgeForm[]]&,
          Arc3D[a,tmin,tmax,h,PlotPoints->ap]/.Line[l_]->l],
				
      Map[VerticalPolygon[#,h,EdgeForm[]]&,
          Arc3D[b,tmin,tmax,h,PlotPoints->ap]/.Line[l_]->l],

	  Polygon[Join[{{a*cmin,a*smin,h}},
				arcah,Reverse[arcbh],{{b*cmin,b*smin,h}}]],
		
	  Polygon[Join[{{a*cmin,a*smin,0}},
				arca0,Reverse[arcb0],{{b*cmin,b*smin,0}}]], 
		
	  VerticalPolygon[{{a*cmin,a*smin},{b*cmin,b*smin}},h],
				
	  VerticalPolygon[{{a*cmax,a*smax},{b*cmax,b*smax}},h]		
				
				}
		
	]
*)	


PolarSolid[{r__},{th__},{h__},opts___Rule]:=
  Graphics3D[PolarRectangularSolid[{r},{th},{h},opts],
    FilterOptions[Graphics3D,opts]//Evaluate,Boxed->False]
(* creates a Graphics3D object from PolarRectangularSolid *)


PolarRiemannSum[
    f_,{r_,rmin_,rmax_,dr_:Automatic},
    {t_,tmin_,tmax_,dt_:Automatic}, opts___]:=
      Module[{ndr=dr,ndt=dt,del=5,as,rsum},
		If[dr===Automatic,ndr=(rmax-rmin)/del];
		If[dt===Automatic,ndt=(tmax-tmin)/del];
		ndr=ndr//N;
		ndt=ndt//N;
		as=ArcSegments /. {opts} /. Options[PolarRiemannSum];
		
		rsum=ndr*ndt*Sum[
			(f/.{r->rr+ndr/2,t->tt+ndt/2})*(rr+ndr/2),
			{rr,rmin,rmax-ndr,ndr},{tt,tmin,tmax-ndt,ndt}];
			
		Print["Riemann Sum = ", rsum];	
		
		grid=PolarGrid3D[{rmin,rmax,ndr},{tmin,tmax,ndt},GridStyle->Hue[1]];
		
		solids=Table[
          PolarRectangularSolid[
          {rr,rr+ndr},{tt,tt+ndt},{0,f/.{r->rr+ndr/2,t->tt+ndt/2}},ArcSegments->as],
		  {rr,rmin,rmax-ndr,ndr},{tt,tmin,tmax-ndt,ndt}]; 
		
		fcn=CircularPlot3D[f,{r,rmin,rmax},{t,tmin,tmax},
          DisplayFunction->Identity,Evaluate[FilterOptions[ParametricPlot3D, opts]]];
		
		leftimage=Show[Graphics3D[solids],Boxed->False,
			DisplayFunction->Identity];
		rightimage=Show[grid,fcn,Boxed->False,
			DisplayFunction->Identity];
		Show[GraphicsArray[{leftimage,rightimage}],
		  FilterOptions[Graphics3D,opts]//Evaluate,
		  DisplayFunction->$DisplayFunction]
		]
		
		
End[]

EndPackage[]